@@ -7,8 +7,15 @@ class ProtocolMetrics(object):
     # TODO Need to enhance when device groups statistics reach
     # more than one page.
 
-    _SUPPORTED_PROTOCOLS_ = ["bgpv4", "bgpv6"]
-    _SKIP = True
+    _SUPPORTED_PROTOCOLS_ = [
+        "bgpv4",
+        "bgpv6",
+        "macsec",
+        "mka",
+        "rocev2_ipv4",
+        "rocev2_ipv6",
+        "isis",
+    ]
 
     _TOPO_STATS = {
         "name": "name",
@@ -22,10 +29,7 @@ class ProtocolMetrics(object):
         "bgpv4": [
             ("name", "Device Group", str),
             ("session_state", "Status", str),
-            # TODO session_flap_count can't be added now
-            # it needs to be added by creating new view.
-            # currently facing an issue in protocol view creation.
-            ("session_flap_count", "Session Flap Count", int, _SKIP),
+            ("session_flap_count", "Session Flap Count", int),
             ("routes_advertised", "Routes Advertised", int),
             ("routes_received", "Routes Rx", int),
             ("route_withdraws_sent", "Routes Withdrawn", int),
@@ -42,8 +46,7 @@ class ProtocolMetrics(object):
         "bgpv6": [
             ("name", "Device Group", str),
             ("session_state", "Status", str),
-            # TODO session_flap_count can't be added now
-            ("session_flap_count", "Session Flap Count", int, _SKIP),
+            ("session_flap_count", "Session Flap Count", int),
             ("routes_advertised", "Routes Advertised", int),
             ("routes_received", "Routes Rx", int),
             ("route_withdraws_sent", "Routes Withdrawn", int),
@@ -57,6 +60,105 @@ class ProtocolMetrics(object):
             ("notifications_sent", "Notifications Tx", int),
             ("notifications_received", "Notifications Rx", int),
         ],
+        "macsec": [
+            ("name", "Device Group", str),
+            ("session_state", "Status", str),
+            ("session_flap_count", "Session Flap Count", int),
+            ("out_pkts_protected", "Protected Packet Tx", int),
+            ("out_pkts_encrypted", "Encrypted Packet Tx", int),
+            ("in_pkts_ok", "Valid Packet Rx", int),
+            ("in_pkts_bad", "Total number of bad Packet Rx", int),
+            ("in_pkts_bad_tag", "Bad Tag/ICV Discarded", int),
+            ("in_pkts_late", "Out of Window Discarded", int),
+            ("in_pkts_no_sci", "Unknown SCI Discarded", int),
+            ("in_pkts_not_using_sa", "Unused SA Discarded", int),
+            ("in_pkts_not_valid", "Invalid ICV Discarded", int),
+            ("in_pkts_unknown_sci", "Unknown SCI Rx", int),
+            ("in_pkts_unused_sa", "Unused SA Rx", int),
+            ("in_pkts_invalid", "Invalid ICV Rx", int),
+            ("in_pkts_untagged", "Non-MACsec Packet Rx", int),
+            ("out_octets_protected", "Tx Bytes Protected", int),
+            ("out_octets_encrypted", "Tx Bytes Encrypted", int),
+            ("in_octets_validated", "Rx Bytes Validated", int),
+            ("in_octets_decrypted", "Rx Bytes Decrypted", int),
+        ],
+        "mka": [
+            ("name", "Device Group", str),
+            ("session_state", "Status", str),
+            ("mkpdu_tx", "MKPDU Tx", int),
+            ("mkpdu_rx", "MKPDU Rx", int),
+            ("live_peer_count", "Live Peer Count", int),
+            ("potential_peer_count", "Potential Peer Count", int),
+            ("latest_key_tx_peer_count", "Latest Key Tx Peer Count", int),
+            ("latest_key_rx_peer_count", "Latest Key Rx Peer Count", int),
+            ("malformed_mkpdu", "Malformed MKPDU", int),
+            ("icv_mismatch", "ICV Mismatch", int),
+        ],
+        "rocev2_ipv4": [
+            ("qp_configured", "QP Configured", int),
+            ("qp_up", "QP Up", int),
+            ("qp_down", "QP Down", int),
+            ("connect_request_tx", "Connect Request Tx", int),
+            ("connect_request_rx", "Connect Request Rx", int),
+            ("connect_reply_tx", "Connect Reply Tx", int),
+            ("connect_reply_rx", "Connect Reply Rx", int),
+            ("ready_tx", "Ready Tx", int),
+            ("ready_rx", "Ready Rx", int),
+            ("disconnect_request_tx", "Disconnect Request Tx", int),
+            ("disconnect_request_rx", "Disconnect Request Rx", int),
+            ("disconnect_reply_tx", "Disconnect Reply Tx", int),
+            ("disconnect_reply_rx", "Disconnect Reply Rx", int),
+            ("reject_tx", "Reject Rx", int),
+            ("reject_rx", "Reject Tx", int),
+            ("unknown_msg_rx", "Unknown MSG Rx", int),
+        ],
+        "rocev2_ipv6": [
+            ("qp_configured", "QP Configured", int),
+            ("qp_up", "QP Up", int),
+            ("qp_down", "QP Down", int),
+            ("connect_request_tx", "Connect Request Tx", int),
+            ("connect_request_rx", "Connect Request Rx", int),
+            ("connect_reply_tx", "Connect Reply Tx", int),
+            ("connect_reply_rx", "Connect Reply Rx", int),
+            ("ready_tx", "Ready Tx", int),
+            ("ready_rx", "Ready Rx", int),
+            ("disconnect_request_tx", "Disconnect Request Tx", int),
+            ("disconnect_request_rx", "Disconnect Request Rx", int),
+            ("disconnect_reply_tx", "Disconnect Reply Tx", int),
+            ("disconnect_reply_rx", "Disconnect Reply Rx", int),
+            ("reject_tx", "Reject Rx", int),
+            ("reject_rx", "Reject Tx", int),
+            ("unknown_msg_rx", "Unknown MSG Rx", int),
+        ],
+        "isis": [
+            ("name", "Device Group", str),
+            ("l1_sessions_up", "L1 Sessions Up", int),
+            ("l1_session_flap", "L1 Session Flap", int),
+            ("l1_database_size", "L1 DB Size", int),
+            ("l1_broadcast_hellos_sent", "L1 Hellos Tx", int),
+            ("l1_broadcast_hellos_received", "L1 Hellos Rx", int),
+            ("l1_point_to_point_hellos_sent", "L1 P2P Hellos Tx", int),
+            ("l1_point_to_point_hellos_received", "L1 P2P Hellos Rx", int),
+            ("l1_psnp_sent", "L1 PSNP Tx", int),
+            ("l1_psnp_received", "L1 PSNP Rx", int),
+            ("l1_csnp_sent", "L1 CSNP Tx", int),
+            ("l1_csnp_received", "L1 CSNP Rx", int),
+            ("l1_lsp_sent", "L1 LSP Tx", int),
+            ("l1_lsp_received", "L1 LSP Rx", int),
+            ("l2_sessions_up", "L2 Sessions Up", int),
+            ("l2_session_flap", "L2 Session Flap", int),
+            ("l2_database_size", "L2 DB Size", int),
+            ("l2_broadcast_hellos_sent", "L2 Hellos Tx", int),
+            ("l2_broadcast_hellos_received", "L2Hellos Rx", int),
+            ("l2_point_to_point_hellos_sent", "L2 P2P Hellos Tx", int),
+            ("l2_point_to_point_hellos_received", "L2 P2P Hellos Rx", int),
+            ("l2_psnp_sent", "L2 PSNP Tx", int),
+            ("l2_psnp_received", "L2 PSNP Rx", int),
+            ("l2_csnp_sent", "L2 CSNP Tx", int),
+            ("l2_csnp_received", "L2 CSNP Rx", int),
+            ("l2_lsp_sent", "L2 LSP Tx", int),
+            ("l2_lsp_received", "L2 LSP Rx", int),
+        ],
     }
 
     _PROTO_NAME_MAP_ = {
@@ -80,6 +182,64 @@ class ProtocolMetrics(object):
             "supported_stats": [s[0] for s in _RESULT_COLUMNS["bgpv6"]],
             "ixn_name": "bgpIpv6Peer",
         },
+        "macsec": {
+            "per_port": "Static MACsec Per Port",
+            "drill_down": "Static MACsec Drill Down",
+            "drill_down_options": [
+                "Static MACsec:Per Session",
+            ],
+            "supported_stats": [s[0] for s in _RESULT_COLUMNS["macsec"]],
+            "ixn_name": "staticMacsec",
+        },
+        "mka": {
+            "per_port": "MKA Per Port",
+            "drill_down": "MKA Drill Down",
+            "drill_down_options": [
+                "MKA:Per Session",
+            ],
+            "supported_stats": [s[0] for s in _RESULT_COLUMNS["mka"]],
+            "ixn_name": "Mka",
+        },
+        "rocev2_ipv4": {
+            "per_port": "RoCEv2 Per Port",
+            "drill_down": "RoCEv2 Drill Down",
+            "drill_down_options": [
+                "RoCEv2:Per Device Group",
+                "RoCEv2:Per Session",
+            ],
+            "supported_stats": [s[0] for s in _RESULT_COLUMNS["rocev2_ipv4"]],
+            "ixn_name": "rocev2",
+        },
+        "rocev2_ipv6": {
+            "per_port": "RoCE6v2 Per Port",
+            "drill_down": "RoCE6v2 Drill Down",
+            "drill_down_options": [
+                "RoCE6v2:Per Device Group",
+                "RoCE6v2:Per Session",
+            ],
+            "supported_stats": [s[0] for s in _RESULT_COLUMNS["rocev2_ipv6"]],
+            "ixn_name": "roce6v2",
+        },
+        "isis": {
+            "per_port": "ISIS-L3 RTR Per Port",
+            "drill_down": "ISIS-L3 RTR Drill Down",
+            "drill_down_options": [
+                "ISIS-L3 RTR:Per Device Group",
+                "ISIS-L3 RTR:Per Session",
+            ],
+            "supported_stats": [s[0] for s in _RESULT_COLUMNS["isis"]],
+            "ixn_name": "isisL3",
+        },
+    }
+
+    _PEER_NAMES = {
+        "bgpv4": "peer_names",
+        "bgpv6": "peer_names",
+        "macsec": "secure_entity_names",
+        "mka": "peer_names",
+        "rocev2_ipv4": "peer_names",
+        "rocev2_ipv6": "peer_names",
+        "isis": "router_names"
     }
 
     def __init__(self, ixnetworkapi):
@@ -193,7 +353,8 @@ class ProtocolMetrics(object):
             if ethernets is None:
                 continue
             for eth in ethernets:
-                port_list.append(eth.get("port_name"))
+                if eth.get("connection") is not None:
+                    port_list.append(eth.get("connection").get("port_name"))
         return port_list
 
     def _do_drill_down(self, view, per_port, row_index, drill_option):
@@ -239,10 +400,9 @@ class ProtocolMetrics(object):
         indices = set(
             [ports.index(p) for p in list(set(config_ports)) if p in ports]
         )
-        drill_option = self._PROTO_NAME_MAP_[protocol]["drill_down_options"][0]
-        drill_option2 = self._PROTO_NAME_MAP_[protocol]["drill_down_options"][
-            1
-        ]
+        drill_options = self._PROTO_NAME_MAP_[protocol].get(
+            "drill_down_options", []
+        )
         drill_name = self._PROTO_NAME_MAP_[protocol]["drill_down"]
         per_port = self._PROTO_NAME_MAP_[protocol]["per_port"]
         column_names = self._RESULT_COLUMNS.get(protocol, [])
@@ -250,8 +410,8 @@ class ProtocolMetrics(object):
         for i in indices:
             try:
                 drill = self.ixn.Statistics.View.find(Caption=drill_name)
-                self._do_drill_down(v, per_port, i, drill_option)
-                self._do_drill_down(v, per_port, i, drill_option2)
+                for option in drill_options:
+                    self._do_drill_down(v, per_port, i, option)
                 self._check_if_page_ready(drill)
             except Exception as e:
                 msg = """
@@ -297,9 +457,13 @@ class ProtocolMetrics(object):
             return
         if skip:
             warn = stat_name in self.columns
-            self._api.warning(
-                "{} metric has no implementation".format(stat_name)
-            ) if warn else None
+            (
+                self._api.warning(
+                    "{} metric has no implementation".format(stat_name)
+                )
+                if warn
+                else None
+            )
             row_dt[stat_name] = (
                 0 if stat_type.__name__ in ["float", "int"] else "na"
             )
@@ -328,10 +492,12 @@ class ProtocolMetrics(object):
         myfilter = [
             {
                 "property": "name",
-                "regex": ".*"
-                if len(self.device_names) == 0
-                else "^%s$"
-                % "|".join(self._api.special_char(self.device_names)),
+                "regex": (
+                    ".*"
+                    if len(self.device_names) == 0
+                    else "^%s$"
+                    % "|".join(self._api.special_char(self.device_names))
+                ),
             }
         ]
         url, payload = self._get_search_payload(
@@ -378,7 +544,9 @@ class ProtocolMetrics(object):
         """
         protocol = request.choice
         request = getattr(request, protocol)
-        self.device_names = request.get("peer_names")
+        peer_name = self._PEER_NAMES[protocol]
+        request.get(peer_name)
+        self.device_names = request.get(peer_name)
         if self.device_names is None:
             self.device_names = []
         self.columns = request.get("column_names")
@@ -387,5 +555,5 @@ class ProtocolMetrics(object):
         if len(self.columns) > 0:
             self.columns.append("name") if "name" not in self.columns else None
             self.columns = list(set(self.columns))
-        with Timer(self._api, "Fetching {} Metrics".format(protocol)):
-            return self._filter_stats(protocol)
+        # with Timer(self._api, "Fetching {} Metrics".format(protocol)):
+        return self._filter_stats(protocol)
